%
%
%  Filaments Angles Statistics Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Filaments Functions">
%        <Item name="Filaments Angles Statistics" icon="Matlab" tooltip="Plot filament angles statistics.">
%          <Command>MatlabXT::XTFilamentsAnglesStatistics(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpFilaments">
%          <Item name="Angles Statistics" icon="Matlab" tooltip="Plot filament angles statistics.">
%            <Command>MatlabXT::XTFilamentsAnglesStatistics(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
% 
%  Description:
%   
%   Plot two histograms that display the statistic of the angles of a filament.
%   The angles are divided in two types: Angles between the segments and
%       angles between the branches. 
%
%

function XTFilamentsAnglesStatistics(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

% get the filament
vFactory = vImarisApplication.GetFactory;
vFilaments = vFactory.ToFilaments(vImarisApplication.GetSurpassSelection);

% search the filament if not previously selected
vSurpassScene = vImarisApplication.GetSurpassScene;
if ~vFactory.IsFilaments(vFilaments)        
    for vChildIndex = 1:vSurpassScene.GetNumberOfChildren
        vDataItem = vSurpassScene.GetChild(vChildIndex - 1);
        if vFactory.IsFilaments(vDataItem)
            vFilaments = vFactory.ToFilaments(vDataItem);
            break;
        end
    end
    % did we find the filament?
    if isequal(vFilaments, [])
        msgbox('Please create some filament!');
        return;
    end
end

vAngle = [];
vBranchAngles = [];

vCount = vFilaments.GetNumberOfFilaments;
vProgressDisplay = waitbar(0,'Computing statistics');
for vFilamentIndex = 0:vCount - 1
  % get the filament coordinates
  vFilamentsXYZ = vFilaments.GetPositionsXYZ(vFilamentIndex);
  vFilamentsEdges = vFilaments.GetEdges(vFilamentIndex) + 1;
  
  vNumberOfSpots = size(vFilamentsXYZ, 1);
  for vSpots = 1:vNumberOfSpots

    [vEdge, vSide] = find(vFilamentsEdges==vSpots);

    vPrevious = -1;
    vNext = [];
    for vNeighborIndex = 1:length(vEdge)
        vNeighbor = vFilamentsEdges(vEdge(vNeighborIndex), 3-vSide(vNeighborIndex));
        if vNeighbor<vSpots
            vPrevious = vNeighbor;
        else
            vNext = [vNext,vNeighbor];            
        end
    end
    
    % angle between previous and next(s)
    if vPrevious>-1 && ~isempty(vNext)
        vVector1 = vFilamentsXYZ(vSpots,:)-vFilamentsXYZ(vPrevious,:);
        vNorm1 = norm(vVector1);
        for vSpot = 1:numel(vNext)
            vVector2 = vFilamentsXYZ(vNext(vSpot),:)-vFilamentsXYZ(vSpots,:);
            vAngle = [vAngle, acos(dot(vVector1,vVector2)/(vNorm1*norm(vVector2)))];
        end
    end

    % angle between next i and next j
    if length(vNext) > 1
        for vSpot1 = 1:length(vNext)-1
            vVector1 = vFilamentsXYZ(vNext(vSpot1),:)-vFilamentsXYZ(vSpots,:);
            vNorm1 = norm(vVector1);
            for vSpot2 = vSpot1+1:length(vNext)
                vVector2 = vFilamentsXYZ(vNext(vSpot2),:)-vFilamentsXYZ(vSpots,:);
                vBranchAngles = [vBranchAngles, ...
                    acos(dot(vVector1,vVector2)/(vNorm1*norm(vVector2)))];
            end
        end    
    end
    
    waitbar((vFilamentIndex+vSpots/vNumberOfSpots)/vCount);
  end
end
close(vProgressDisplay);

vAngle = vAngle*180/pi;
vBranchAngles = vBranchAngles*180/pi;

subplot(2,1,1);
vSize = 2;
hist(vAngle,vSize/2+vSize*(0:max(ceil(vAngle/vSize))));
title('Angles between segments');
xlabel('Angle');
ylabel('# of segments');


subplot(2,1,2);
vSize = 5;
hist(vBranchAngles,vSize/2+vSize*(0:max(ceil(vBranchAngles/vSize))));
title('Angles between branches');
xlabel('Angle');
ylabel('# of branches');

